/*Copyright (C) 2019 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana

This file is part of Plataforma Integrada MEC.

Plataforma Integrada MEC is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Plataforma Integrada MEC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with Plataforma Integrada MEC.  If not, see <http://www.gnu.org/licenses/>.*/

import React, { useState } from 'react'
import LinkIcon from '../../img/link_icon.svg'
import ChooseLink from './ChooseLinkSection.js'
import { WrapperBox, BlueButton, GrayButton } from './StyledComponents.js';
import { DottedBox } from './StyledComponents.js';
import { getAxiosConfigFromJSON, updateHeaders, deleteRequest, putRequest } from '../HelperFunctions/getAxiosConfig.js'
import AttachFileIcon from '@material-ui/icons/AttachFile';
import axios from 'axios'
import { apiUrl } from '../../env';
import DoneIcon from '@material-ui/icons/Done';
import DeleteIcon from '@material-ui/icons/Delete';

import SnackBar from '../SnackbarComponent';

export default function UploadFileWrapper(props) {
    /*-----------------------------------------------------------------
        - Wrapper for file upload box
        - has three different stages:
            - Default: prompts user to select a file or a link
            - Choosing Link: displayed when the user selects "ENVIAR LINK"; renders "ChooseLinkSection.js"
            - file selected: displayed when the user has chosen a file to upload. this section shows the upload progress and lets the user delete the current object and return to default stage
            - error: displayed when an error occurs during the file upload.

        - Props used:
            - submit : called when the user clicks "ENVIAR" inside ChooseLinkSection; renders the alert snackbar to let them know the link was submitted
            - contrast : used to display contrast colors or not
            */

    const [stage, setStage] = useState(props.prevFile ? "fileSelected" : "default")
    const handleNextStage = (newStage) => { setStage(newStage) }

    // eslint-disable-next-line
    const [fileToUpload, setFileToUpload] = useState(null);
    const chunkSize = 262144
    const uploadUrl = `${apiUrl}/learning_objects/` + props.draftID + '/chunk'
    const [fileName, setFileName] = useState(props.prevFile ? props.prevFile.name : null)
    const [fileDoneUploading, setFileDoneUploading] = useState(props.prevFile ? true : false)
    const [uploadProgress, setProgress] = useState(0)
    const [attachmentID, setAttachmentID] = useState(props.prevFile ? props.prevFile.id : null)

    async function onFileChange(file) {
        if (!file) return;

        let newFile = file
        //console.log(newFile)
        setFileToUpload(newFile);
        setFileName(newFile.name)

        let total = Math.ceil(newFile.size / chunkSize)
        let currentChunkStartByte = 0;
        let currentChunkFinalByte = chunkSize > newFile.size ? newFile.size : chunkSize;
        let chunkIdentifier = props.draftID + '-' + newFile.name;
        let remainingBytes = 0
        handleNextStage("fileSelected")
        for (var i = 0; i < total; i++) {
            let chunk = newFile.slice(currentChunkStartByte, currentChunkFinalByte)
            let config = getAxiosConfigFromJSON()
            let formData = new FormData()
            formData.append('_chunkFilename', newFile.name);
            formData.append('_chunkIdentifier', chunkIdentifier);
            formData.append('_totalChunks', total);
            formData.append('_chunkSize', chunkSize);
            formData.append('_currentChunkSize', chunk.size);
            formData.append('_chunkNumber', i);
            formData.append('_totalSize', newFile.size);
            formData.append('file', chunk);

            try {
                const response = await axios.post(uploadUrl, formData, config);
                //console.log(response)
                if (response.headers['access-token']) {
                    updateHeaders(response.headers)
                }
                setProgress(Math.round((currentChunkFinalByte / newFile.size) * 100))
                remainingBytes = newFile.size - currentChunkFinalByte;
                if (currentChunkFinalByte === newFile.size) {
                    setFileDoneUploading(true)
                    setAttachmentID(response.data.id)
                }
                else if (remainingBytes < chunkSize) {
                    currentChunkStartByte = currentChunkFinalByte;
                    currentChunkFinalByte = currentChunkStartByte + remainingBytes;
                }
                else {
                    currentChunkStartByte = currentChunkFinalByte;
                    currentChunkFinalByte = currentChunkStartByte + chunkSize;
                }
            } catch (error) {
                handleNextStage("error")
                return;
            }
        }
    };

    const handleDelete = () => {
        if (attachmentID != null) {
            const url = `/learning_objects/${props.draftID}/attachment/${attachmentID}`

            deleteRequest(url, (data) => { handleNextStage("default") }, (error) => { console.log(error) })
        }
    }

    const handleCancel = () => {

    }

    const handleDragOver = e => {
        e.preventDefault();
        e.stopPropagation();
        e.dataTransfer.dropEffect = 'copy';
    };

    const handleDrop = e => {
        e.preventDefault();
        e.stopPropagation();
        let files = [...e.dataTransfer.files]

        if (files && files.length > 0) {
            onFileChange(files[0])
        }
    };

    const [snackbarOpen, toggleSnackbar] = useState(false)

    const handleChooseLink = (link) => {
        const url = `/learning_objects/${props.draftID}`

        let payload = {
            "learning_object": {
                "id": props.draftID,
                "link": link
            }
        }

        putRequest(url, payload, (data) => { toggleSnackbar(true) }, (error) => { console.log(error) })
    }

    switch (stage) {
        case "error":
            return (
                <WrapperBox contrast={props.contrast}>
                    <div className="inner">
                        <div className="upload-title">Erro</div>
                        <span>Clique no botão para tentar novamente.</span>
                        <div className="flex-column">
                            <div className="buttons-div">
                                <GrayButton contrast={props.contrast} onClick={() => { handleNextStage("default") }}>
                                    <span className="button-text">
                                        Voltar
                                    </span>
                                </GrayButton>
                            </div>
                        </div>
                    </div>
                </WrapperBox>
            )
        case "fileSelected":
            return (
                <WrapperBox contrast={props.contrast}>
                    <div className="inner">
                        <div className="upload-title">
                            {fileDoneUploading ? 'O arquivo foi carregado' : 'Carregando arquivo'}
                        </div>

                        <div className="uploading">
                            <div className="upload-item">
                                <div className="item-info">
                                    {
                                        fileDoneUploading ?
                                            (
                                                <React.Fragment>
                                                    <div className="file-status">
                                                        <DoneIcon contrast={props.contrast} className="icon icon-margin" /> {fileName}
                                                    </div>
                                                    <GrayButton contrast={props.contrast} onClick={() => { handleDelete() }}>
                                                        Excluir <DeleteIcon className="icon icon-remove" />
                                                    </GrayButton>
                                                </React.Fragment>
                                            )
                                            :
                                            (
                                                <React.Fragment>
                                                    <div className="file-status">
                                                        {uploadProgress}% {fileName}
                                                    </div>
                                                    <GrayButton contrast={props.contrast} onClick={() => { handleCancel() }}>
                                                        Cancelar <DeleteIcon className="icon icon-remove" />
                                                    </GrayButton>
                                                </React.Fragment>
                                            )
                                    }
                                </div>
                            </div>
                            <div className="warning">
                                <span>Não se esqueça de preencher as</span>
                                <br />
                                <span>informações sobre o recurso ao lado.</span>
                            </div>
                        </div>
                    </div>
                </WrapperBox>
            )
        case "choosingLink":
            return (
                <React.Fragment>
                    <SnackBar
                        snackbarOpen={snackbarOpen}
                        handleClose={() => { toggleSnackbar(false) }}
                        severity={"info"}
                        color={"#00acc1"}
                        text={"Link salvo com sucesso!"}
                    />
                    <ChooseLink contrast={props.contrast} handleNextStage={handleNextStage} submit={handleChooseLink} />
                </React.Fragment>
            )
        default:
            return (
                <WrapperBox contrast={props.contrast}>
                    <div className="inner">
                        <div className="upload-title"> Enviar Recurso</div>
                        <div className="flex-column">

                            <div className="upload-container">
                                <div className="upload-form">
                                    <form id="file_upload">
                                        <DottedBox
                                            contrast={props.contrast}
                                            onDrop={e => handleDrop(e)}
                                            onDragOver={e => handleDragOver(e)}
                                        >
                                            <AttachFileIcon className="icon" />
                                            <input
                                                type="file"
                                                onChange={(e) => { onFileChange(e.target.files[0]) }}
                                                id="upload-file"
                                                style={{ display: "none" }}
                                            />
                                            <BlueButton contrast={props.contrast}>
                                                <label htmlFor="upload-file" style={{ width: "inherit", cursor: "pointer" }}>
                                                    ESCOLHER ARQUIVO
                                                </label>
                                            </BlueButton>
                                            <span style={{ marginTop: "6px" }}>Ou arrastar e soltar o arquivo aqui</span>
                                        </DottedBox>
                                    </form>
                                </div>
                            </div>

                            <div className="strike-box">
                                <div className="strike" /><h3>ou</h3><div className="strike" />
                            </div>

                            <div className="enviar-link-texto">
                                <img alt="" src={LinkIcon} />
                                <br />
                                <span>Enviar link de um recurso de outro site</span>
                            </div>

                            <BlueButton contrast={props.contrast} onClick={() => { handleNextStage("choosingLink") }}>ENVIAR LINK</BlueButton>
                        </div>
                    </div>
                </WrapperBox>
            )
    }

}
